// sharedPtr1.cpp
// Konstruktoren und Destruktor der Klasse shared_ptr<T>
// -----------------------------------------------------
#include <iostream>
#include <memory>
using namespace std;

struct Base { void msg() {  cout << "Base-Objekt!" << endl; }  };

struct Derived : public Base
{  void msg() { cout << "Derived-Objekt!" << endl; }
   ~Derived() { cout << "Derived-Destruktor ausgefuehrt!" << endl; }
};

void test( shared_ptr<Base> pB)      // Kopierkonstruktor
{
   pB->msg();                        // Base::msg()
   cout << "Es gibt " << pB.use_count() << " Besitzer." << endl;
}

int main()
{
   shared_ptr<int> sp1(new int(10));
   shared_ptr<int> sp2(sp1);           // Kopierkonstruktor.
                                       // Beide zeigen auf das selbe Objekt.
   *sp1 += 10;
   cout << "*sp2 = " << *sp2 << endl;  // *sp2 = 20

   cout << "sp1.use_count() = " << sp1.use_count() << endl;  // 2
   sp1.reset();                        // Objekt existiert noch.
   cout << "sp1.use_count() = " << sp1.use_count() << endl;  // 0
   cout << "sp2.use_count() = " << sp2.use_count() << endl;  // 1

   shared_ptr<int> sp3(move(sp2));     // Move-Konstruktor.
   if( sp3.unique())                   // sp3.unique() == true  
      cout << "sp3 alleiniger Besitzer: " << *sp3 << endl;
   sp3.reset();                        // Gibt Speicher frei.

   shared_ptr<Base> pBase(new Derived);        // oK!
   shared_ptr<Derived> pDerived(new Derived);  // oK!
// shared_ptr<Derived> pBase(new Base);        // Fehler!

   pBase->msg();                       // Base::msg()
   pBase.reset();                      // Derived-Destruktor! 
   test(pDerived);                     // ok!
   pDerived->msg();                    // Derived:msg()
   return 0;
}

/* Ausgabe:

*sp2 = 20
sp1.use_count() = 2
sp1.use_count() = 0
sp2.use_count() = 1
sp3 alleiniger Besitzer: 20
Base-Objekt!
Derived-Destruktor ausgefuehrt!
Base-Objekt!
Es gibt 2 Besitzer.
Derived-Objekt!
Derived-Destruktor ausgefuehrt!
*/
