// ClientServer.h
// Definiert die Klassen ClientServerStream, Client und Server
// zur Kommunikation zwischen einem Client und einem Server
// -------------------------------------------------------
#include <thread>
#include <mutex>
#include <condition_variable>
#include <string>
#include <iostream>
using namespace std;

class ClientServerStream
{
    int    requ;       // Anfragenummer
    string resp;       // Antwort

    bool busyServer;   // true: Server aktiv, 
                       // false: Client aktiv.
    bool done;         // Setzt der Client nach dem letzten 
                       // Lesen auf true.
    mutex mtx;
    condition_variable cond; 

  public:
    ClientServerStream() : requ(0), resp(""), 
                           busyServer(false), done(false){}

    void request(int n)
    {
        unique_lock< mutex> lock(mtx);

        while(busyServer)           // Falls Server aktiv:
           cond.wait(lock);         // warten.

        requ = n;                   // Anfrage stellen.
        busyServer = true;          // Beantwortung zulassen.
        cond.notify_one();          // Server aufwecken.

        while(busyServer)           // Warten auf Antwort:
            cond.wait(lock);
                                    // Antwort auswerten:
        cout << "\nAuswertung der Antwort im Client: " 
             << resp << endl;
                                    // Nchste Anfrage
        cond.notify_one();          // zulassen.
    }
 
    void respond()
    {
       unique_lock<mutex> lock(mtx);

       while(!busyServer && !done )   // Falls Client aktiv: 
           cond.wait(lock);           // warten.

       if(done)                // Falls letzte Frage bereits
           return;             // beantwortet.

       switch( requ )
       {
           case 1: resp = " Anfrage hat Nummer 1\n";
                   break;
           case 2: resp = " Anfrage hat Nummer 2\n";
                   break;
           case 3: resp = " Anfrage hat Nummer 3\n";
                   break;
           default: resp = "Anfragenummer existiert nicht\n";
       }

       busyServer = false;     // Server fertig.
       cond.notify_one();      // Client aufwecken.
     }

     void setDone(){ done = true;}
     bool isDone(){ return done; }
};

class Client
{
     ClientServerStream& buf;

  public:
     Client(ClientServerStream& cs) : buf(cs) {}

     void operator()(int n)     // n ist die Anfragenummer
     {
         for(int i=1; i <= n; i++)
            buf.request(i);

         buf.setDone();
     }
};

class Server
{
     ClientServerStream& buf;

  public:
     Server(ClientServerStream& cs) : buf(cs) {}

     void operator()()
     {
         while(!buf.isDone())
             buf.respond();
     }
};
