// -------------------------------------------------------
// Fraction.cpp
// Definition der Methoden und friend-Funktionen,
// die nicht inline sind.
// -------------------------------------------------------

#include "Fraction.h"

// Konstruktoren:
Fraction::Fraction(long z, long n)
{
   if(n == 0)
   {
     cerr << "\nFehler: Division durch Null!\n";
     exit(1);
   }
   if( n < 0 )
      z = -z, n = -n;
   zaehler = z;
   nenner  = n;
}
Fraction::Fraction( double x)
{
   x *= 1000.0;
   x += (x>=0.0) ? 0.5 : -0.5;    // Die 4. Stelle runden.
   zaehler = (long)x;
   nenner = 1000;
   kuerzen();
}

Fraction operator+(const Fraction& a, const Fraction& b )
{
   Fraction temp;

   temp.nenner = a.nenner * b.nenner;
   temp.zaehler = a.zaehler*b.nenner + b.zaehler * a.nenner;
   temp.kuerzen();
   return temp;
}

Fraction operator-(const Fraction& a, const Fraction& b )
{
   Fraction temp = a;

   temp += (-b);

   return temp;
}

Fraction operator*(const Fraction& a, const Fraction& b )
{
   Fraction temp;

   temp.zaehler = a.zaehler * b.zaehler;
   temp.nenner  = a.nenner  * b.nenner;
   temp.kuerzen();

   return temp;
}

Fraction operator/(const Fraction& a, const Fraction& b )
{
   if( b.zaehler == 0)
   {
     cerr << "\nFehler: Division durch Null!\n";
     exit(1);
   }

   Fraction temp;             // a mit dem Kehrwert von b multiplizieren
   temp.zaehler = a.zaehler * b.nenner;
   temp.nenner  = a.nenner  * b.zaehler;

   if( temp.nenner < 0 )
     temp.zaehler = -temp.zaehler,
     temp.nenner  = -temp.nenner;

   temp.kuerzen();

   return temp;
}

ostream& operator<<(ostream& os, const Fraction& a)
{
  os << a.zaehler << "/" << a.nenner;
  return os;
}

istream& operator>>(istream& is, Fraction& a)
{
  cout << "Bruch eingeben:\n"
          "  Zaehler:      ";   is >> a.zaehler;
  cout << "  Nenner != 0:  ";   is >> a.nenner;

  if( !is) return is;

  if( a.nenner == 0)
  {
     cout << "\nFehler: Der Nenner ist 0\n"
             "  Neuer Nenner != 0: ";   is >> a.nenner;

     if( a.nenner == 0)
     {
       cerr << "\nFehler: Division durch Null!\n";
       exit(1);
     }
  }
  if( a.nenner < 0 )
     a.zaehler = -a.zaehler,
     a.nenner  = -a.nenner;

  return is;
}

void Fraction::kuerzen()
{
    // Zaehler und Nenner durch den grten gemeinsamen
    // Teiler dividieren.

    if( zaehler == 0)
    {
        nenner = 1;
        return;
    }
    // Berechnung des grten gemeinsamen Teilers
    // nach einem Algorithmus von Euklid.
    long a = (zaehler < 0) ? -zaehler : zaehler,
         b = nenner, 
         hilf;

    while( b != 0)
    {
        hilf = a % b;  a = b;  b = hilf;
    }
    // a ist jetzt der grte gemeinsame Teiler

    zaehler /= a;
    nenner  /= a;
}
