// ------------------------------------------------------------
//  bitmap.h : Definition des Klassen-Templates Bitmap<NX,NY>
//             zur Darstellung von Rastergraphiken. Geraden
//             werden mit dem Bresenham-Algorithmus gezeichnet.
// ------------------------------------------------------------

#ifndef _BITMAP_
#define _BITMAP_

#include <bitset>
#include <stdexcept>
using namespace std;

template <int NX, int NY>  // Anzahl Pixel in x- / y-Richtung
class Bitmap : public bitset<NX*NY>
{
    private:
        int ax, ay;       // Aktuelle Cursorposition

    public:
        Bitmap();
        void moveTo(int x, int y);
        void lineTo(int x, int y);
        size_t xSize() const { return NX; }
        size_t ySize() const { return NY; }
};

template <int NX, int NY>
Bitmap<NX,NY>::Bitmap()
{
   this->reset();      // Alle Bits auf 0
   ax = 0;             // aktuelle Position
   ay = 0;             // in x-/y-Richtung
}

template <int NX, int NY>
void Bitmap<NX,NY>::moveTo(int x, int y)
{
  if( x >= 0 && x < NX && y >= 0 && y < NY)
  {
     ax = x;  ay = y;
  }
  else throw invalid_argument("Ungueltiges Argument\n");
}

// Bresenham-Algorithmus:
template <int NX, int NY>
void Bitmap<NX,NY>::lineTo(int x, int y)
{
  if( x >= 0 && x < NX && y >= 0 && y < NY)
  {
    int savex = x, savey = y;
    if(ax > x)   // In aufsteigender x-Richtung zeichnen
    {            // => (ax,ay) und (x,y) ggfs. tauschen
      int temp = ax; ax = x; x = temp;
      temp = ay; ay = y; y = temp;
    }
    int dx = x - ax, dy = y - ay;
    int xinc = 1, yinc = 1;    // Zum Inkrementieren

    if( dy < 0)                // Steigung < 0 ?
    { dy = -dy; yinc = -1; }   // in y-Richtung dekrementieren.

    int count = dx + dy;       // Anzahl zu setzender Pixel

    int d = (dx - dy)/2;       // Bestimmt die Richtung fr
                               // nchstes Pixel (s. Schleife).
    while( count-- > 0)
    {
      int i = ax + ay * NX;    // Index im Bitset
      this->set(i);            // Bit setzen

      if( d < 0)               // Nchstes Pixel in
      { ay += yinc;  d += dx; }    // y-Richtung
      else                         // sonst in
      { ax += xinc;  d -= dy; }    // x-Richtung
    }

    ax = savex; ay = savey;    // Aktuelle Cursor-Position
  }
  else throw invalid_argument("Ungltiges Argument\n");
}

#endif
