//----------------------------------------------------- 
// stack.h : Das Klassen-Template Stack mit 
//           den Methoden push() und pop().
//----------------------------------------------------- 
#ifndef STACK_H_
#define STACK_H_

template<class T>
class Stack
{
  private:
    T* basePtr;      // Zeiger auf Vektor
    int tip;         // Stack-Spitze
    int max;         // maximale Anzahl Elemente
  public:
    Stack( int n ){ basePtr = new T[n]; max = n; tip = 0; }
    Stack( const Stack<T>&);     
    ~Stack(){ delete[] basePtr; }
                                 
    Stack<T>& operator=( const Stack<T>& );

    bool empty(){ return (tip == 0); }
    bool push( const T& x);
    bool pop(T& x);
};

// Definition der Methoden push() und pop()
 
template<class T>
bool Stack<T>::push( const T& x)
{ 
   if(tip <= max - 1)             // Falls noch Platz
   { 
       basePtr[tip++] = x; 
       return true;
   }
   else return false;
}
 
template<class T>
bool Stack<T>::pop( T& x)
{
   if(tip > 0)                   // Falls Stack nicht leer
   {
       x = basePtr[--tip];
       return true;
   }
   else return false;
}
 
template<class T>
Stack<T>::Stack( const Stack<T>& stk)
{
   tip = stk.tip;
   max = stk.max;

   basePtr = new T[max];
   
   for(int i = 0; i < tip; i++)
       basePtr[i] = stk.basePtr[i];
}

template<class T>
Stack<T>&  Stack<T>::operator=( const Stack<T>& stk)
{
   if( this != &stk)
   {
       tip = stk.tip;
       max = stk.max;

       delete[] basePtr;
       basePtr = new T[max];
   
       for(int i = 0; i < tip; i++)
           basePtr[i] = stk.basePtr[i];
   }
   return *this;
}

 
// --- Spezialisierung fr C-Strings: ---
template<>
class Stack<char*>
{
  private:
     char** basePtr;  // Zeiger auf Vektor von char-Zeigern
     int tip, max;    // Stack-Spitze, max. Anzahl Elemente
  public:  
   // Konstruktor, Destruktor, Kopierkonstruktor,
   // Zuweisungsoperator, push(), pop(), empty()
    Stack( int n ){ basePtr = new char*[n]; max = n; tip = 0; }
    ~Stack(){ delete[] basePtr; }
    bool empty(){ return (tip == 0); }
    bool push( char* x);
    bool pop(char* &x);
  private:
    Stack( const Stack<char*>&);
    Stack<char*>& operator=( const Stack<char*>& );
};

bool Stack<char*>::push( char* x)
{ 
   if(tip <= max - 1)             // Falls noch Platz
   { 
       basePtr[tip++] = x; 
       return true;
   }
   else return false;
}

bool Stack<char*>::pop( char* &x)
{
   if(tip > 0)                   // Falls Stack nicht leer
   {
       x = basePtr[--tip];
       return true;
   }
   else return false;
}


#include <iostream>

inline void testStr()
{
    Stack<char*> strStk(10);
    strStk.push((char*)"Hallo");
    strStk.push((char*)"Guten Morgen");

    char* str;
    while( strStk.pop(str))
       std::cout << str << endl;
}

#endif   // STACK_H_
