// producer_consumer.h
// Demonstriert die Verwendung von Bedingungsvariablen
// --------------------------------------------------------
#include <thread>
#include <mutex>
#include <condition_variable>
#include <ctime>       // Fuer time()
#include <cstdlib>     // Fuer rand(), srand() und NULL
#include <iostream>
using namespace std;

class Buffer
{
   private:
    unsigned buf;
    bool empty;    // true, falls Puffer leer, sonst false.
    bool done;     // Nach letzter Schreiboperation true

    mutex mtx;
    condition_variable cond;

   public:
    Buffer(): buf(0), empty(true), done(false){}

    void put(unsigned val)
    {
      unique_lock<mutex> lock(mtx);

      while(!empty)          // Solange Puffer nicht leer:
        cond.wait(lock);     // schlafen.

      buf = val;             // Puffer leer: schreiben.
      empty = false;         // Puffer nicht mehr leer.
      cond.notify_one();     // Nchsten Thread aufwecken.
    }

    unsigned get() 
    {
      unique_lock<mutex> lock(mtx);

      while(empty)           // Warten, falls Puffer leer.
        cond.wait(lock);

      unsigned res = buf;    // Puffer lesen.   
      empty = true;          // Puffer wieder leer.
      cond.notify_one();     // Nchsten Thread aufwecken.

      return res;
    }

    void setDone()
    {
        unique_lock<mutex> lock(mtx);
        while(!empty)            // Solange Puffer nicht leer:
            cond.wait(lock);     // schlafen.
        done = true;
        cond.notify_one();
    }
    bool isDone() const{ return done; }
};

// Die Klassen Producer und Consumer
// -------------------------------------------

class Producer
{
  private:
   Buffer& bufref;
  public:
   Producer(Buffer& b):bufref(b)   
   { srand((unsigned)time(NULL)); }

   void operator()()
   {
     for(int i=0; i < 10; i++)  bufref.put(1+10*i);
     bufref.setDone();       // Schreibvorgang beendet
   }
};

class Consumer
{
  private:
    Buffer& bufref;
   public:
     Consumer(Buffer& b) : bufref(b){}

     void operator()()
    {
      int val;
      while(!bufref.isDone())
     {
       val = bufref.get();
       cout << "Customer liest " << val << endl;
     }
  }
};
